@@ -1,17 +1,12 @@
-# coding: utf-8
 """Wrappers for forwarding stdout/stderr over zmq"""
 
 # Copyright (c) IPython Development Team.
 # Distributed under the terms of the Modified BSD License.
 
-from __future__ import print_function
 import atexit
 from binascii import b2a_hex
 from collections import deque
-try:
-    from importlib import lock_held as import_lock_held
-except ImportError:
-    from imp import lock_held as import_lock_held
+from imp import lock_held as import_lock_held
 import os
 import sys
 import threading
@@ -19,13 +14,16 @@ import warnings
 from io import StringIO, TextIOBase
 
 import zmq
-from zmq.eventloop.ioloop import IOLoop
+if zmq.pyzmq_version_info() >= (17, 0):
+    from tornado.ioloop import IOLoop
+else:
+    # deprecated since pyzmq 17
+    from zmq.eventloop.ioloop import IOLoop
 from zmq.eventloop.zmqstream import ZMQStream
 
 from jupyter_client.session import extract_header
 
 from ipython_genutils import py3compat
-from ipython_genutils.py3compat import unicode_type
 
 #-----------------------------------------------------------------------------
 # Globals
@@ -296,6 +294,7 @@ class OutStream(TextIOBase):
         self.parent_header = {}
         self._master_pid = os.getpid()
         self._flush_pending = False
+        self._subprocess_flush_pending = False
         self._io_loop = pub_thread.io_loop
         self._new_buffer()
         self.echo = None
@@ -362,6 +361,7 @@ class OutStream(TextIOBase):
         unless the thread has been destroyed (e.g. forked subprocess).
         """
         self._flush_pending = False
+        self._subprocess_flush_pending = False
 
         if self.echo is not None:
             try:
@@ -377,8 +377,8 @@ class OutStream(TextIOBase):
             # since pub_thread is itself fork-safe.
             # There should be a better way to do this.
             self.session.pid = os.getpid()
-            content = {u'name':self.name, u'text':data}
-            self.session.send(self.pub_thread, u'stream', content=content,
+            content = {'name':self.name, 'text':data}
+            self.session.send(self.pub_thread, 'stream', content=content,
                 parent=self.parent_header, ident=self.topic)
 
     def write(self, string):
@@ -394,18 +394,20 @@ class OutStream(TextIOBase):
             raise ValueError('I/O operation on closed file')
         else:
             # Make sure that we're handling unicode
-            if not isinstance(string, unicode_type):
+            if not isinstance(string, str):
                 string = string.decode(self.encoding, 'replace')
 
             is_child = (not self._is_master_process())
             # only touch the buffer in the IO thread to avoid races
             self.pub_thread.schedule(lambda : self._buffer.write(string))
             if is_child:
-                # newlines imply flush in subprocesses
                 # mp.Pool cannot be trusted to flush promptly (or ever),
                 # and this helps.
-                if '\n' in string:
-                    self.flush()
+                if self._subprocess_flush_pending:
+                    return
+                self._subprocess_flush_pending = True
+                # We can not rely on self._io_loop.call_later from a subprocess
+                self.pub_thread.schedule(self._flush)
             else:
                 self._schedule_flush()
 
@@ -424,7 +426,7 @@ class OutStream(TextIOBase):
 
         This should only be called in the IO thread.
         """
-        data = u''
+        data = ''
         if self._buffer is not None:
             buf = self._buffer
             self._new_buffer()
