@@ -3,8 +3,6 @@
 # Copyright (c) IPython Development Team.
 # Distributed under the terms of the Modified BSD License.
 
-from __future__ import print_function
-
 import atexit
 import os
 import sys
@@ -13,7 +11,9 @@ import signal
 import traceback
 import logging
 
+import tornado
 from tornado import ioloop
+
 import zmq
 from zmq.eventloop import ioloop as zmq_ioloop
 from zmq.eventloop.zmqstream import ZMQStream
@@ -72,6 +72,10 @@ kernel_flags.update({
         {'IPKernelApp' : {'pylab' : 'auto'}},
         """Pre-load matplotlib and numpy for interactive use with
         the default matplotlib backend."""),
+    'trio-loop' : (
+        {'InteractiveShell' : {'trio_loop' : False}},
+        'Enable Trio as main event loop.'
+    ),
 })
 
 # inherit flags&aliases for any IPython shell apps
@@ -147,6 +151,7 @@ class IPKernelApp(BaseIPythonApplication, InteractiveShellApp,
     # streams, etc.
     no_stdout = Bool(False, help="redirect stdout to the null device").tag(config=True)
     no_stderr = Bool(False, help="redirect stderr to the null device").tag(config=True)
+    trio_loop = Bool(False, help="Set main event loop.").tag(config=True)
     quiet = Bool(True, help="Only send stdout/stderr to output stream").tag(config=True)
     outstream_class = DottedObjectName('ipykernel.iostream.OutStream',
         help="The importstring for the OutStream factory").tag(config=True)
@@ -378,12 +383,12 @@ class IPKernelApp(BaseIPythonApplication, InteractiveShellApp,
             e_stderr = None if self.quiet else sys.__stderr__
 
             sys.stdout = outstream_factory(self.session, self.iopub_thread,
-                                           u'stdout',
+                                           'stdout',
                                            echo=e_stdout)
             if sys.stderr is not None:
                 sys.stderr.flush()
             sys.stderr = outstream_factory(self.session, self.iopub_thread,
-                                           u'stderr',
+                                           'stderr',
                                            echo=e_stderr)
         if self.displayhook_class:
             displayhook_factory = import_item(str(self.displayhook_class))
@@ -518,7 +523,7 @@ class IPKernelApp(BaseIPythonApplication, InteractiveShellApp,
         FIXME: if/when tornado supports the defaults in asyncio,
                remove and bump tornado requirement for py38
         """
-        if sys.platform.startswith("win") and sys.version_info >= (3, 8):
+        if sys.platform.startswith("win") and sys.version_info >= (3, 8) and tornado.version_info < (6, 1):
             import asyncio
             try:
                 from asyncio import (
@@ -534,6 +539,20 @@ class IPKernelApp(BaseIPythonApplication, InteractiveShellApp,
                     # fallback to the pre-3.8 default of Selector
                     asyncio.set_event_loop_policy(WindowsSelectorEventLoopPolicy())
 
+    def init_pdb(self):
+        """Replace pdb with IPython's version that is interruptible.
+
+        With the non-interruptible version, stopping pdb() locks up the kernel in a
+        non-recoverable state.
+        """
+        import pdb
+        from IPython.core import debugger
+        if hasattr(debugger, "InterruptiblePdb"):
+            # Only available in newer IPython releases:
+            debugger.Pdb = debugger.InterruptiblePdb
+            pdb.Pdb = debugger.Pdb
+            pdb.set_trace = debugger.set_trace
+
     @catch_config_error
     def initialize(self, argv=None):
         self._init_asyncio_patch()
@@ -541,6 +560,7 @@ class IPKernelApp(BaseIPythonApplication, InteractiveShellApp,
         if self.subapp is not None:
             return
 
+        self.init_pdb()
         self.init_blackhole()
         self.init_connection_file()
         self.init_poller()
@@ -579,10 +599,19 @@ class IPKernelApp(BaseIPythonApplication, InteractiveShellApp,
             self.poller.start()
         self.kernel.start()
         self.io_loop = ioloop.IOLoop.current()
-        try:
-            self.io_loop.start()
-        except KeyboardInterrupt:
-            pass
+        if self.trio_loop:
+            from ipykernel.trio_runner import TrioRunner
+            tr = TrioRunner()
+            tr.initialize(self.kernel, self.io_loop)
+            try:
+                tr.run()
+            except KeyboardInterrupt:
+                pass
+        else:
+            try:
+                self.io_loop.start()
+            except KeyboardInterrupt:
+                pass
 
 
 launch_new_instance = IPKernelApp.launch_instance
