@@ -6,10 +6,7 @@
 import re
 import sys
 from distutils.version import LooseVersion as V
-try:
-    from queue import Empty  # Py 3
-except ImportError:
-    from Queue import Empty  # Py 2
+from queue import Empty
 
 import nose.tools as nt
 from nose.plugins.skip import SkipTest
@@ -17,9 +14,9 @@ from nose.plugins.skip import SkipTest
 from traitlets import (
     HasTraits, TraitError, Bool, Unicode, Dict, Integer, List, Enum
 )
-from ipython_genutils.py3compat import string_types, iteritems
 
-from .utils import TIMEOUT, start_global_kernel, flush_channels, execute
+from .utils import (TIMEOUT, start_global_kernel, flush_channels, execute,
+                   get_reply, )
 
 #-----------------------------------------------------------------------------
 # Globals
@@ -99,14 +96,14 @@ class MimeBundle(Reference):
     metadata = Dict()
     data = Dict()
     def _data_changed(self, name, old, new):
-        for k,v in iteritems(new):
+        for k,v in new.items():
             assert mime_pat.match(k)
-            assert isinstance(v, string_types)
+            assert isinstance(v, str)
 
 
 # shell replies
 class Reply(Reference):
-    status = Enum((u'ok', u'error'), default_value=u'ok')
+    status = Enum(('ok', 'error'), default_value='ok')
 
 
 class ExecuteReply(Reply):
@@ -143,7 +140,7 @@ class ArgSpec(Reference):
 
 
 class Status(Reference):
-    execution_state = Enum((u'busy', u'idle', u'starting'), default_value=u'busy')
+    execution_state = Enum(('busy', 'idle', 'starting'), default_value='busy')
 
 
 class CompleteReply(Reply):
@@ -183,7 +180,7 @@ class CommInfoReply(Reply):
 
 
 class IsCompleteReply(Reference):
-    status = Enum((u'complete', u'incomplete', u'invalid', u'unknown'), default_value=u'complete')
+    status = Enum(('complete', 'incomplete', 'invalid', 'unknown'), default_value='complete')
 
     def check(self, d):
         Reference.check(self, d)
@@ -208,7 +205,7 @@ class Error(ExecuteReplyError):
 
 
 class Stream(Reference):
-    name = Enum((u'stdout', u'stderr'), default_value=u'stdout')
+    name = Enum(('stdout', 'stderr'), default_value='stdout')
     text = Unicode()
 
 
@@ -275,7 +272,7 @@ def test_execute():
     flush_channels()
 
     msg_id = KC.execute(code='x=1')
-    reply = KC.get_shell_msg(timeout=TIMEOUT)
+    reply = get_reply(KC, msg_id, TIMEOUT)
     validate_message(reply, 'execute_reply', msg_id)
 
 
@@ -357,10 +354,10 @@ def test_user_expressions():
 
     msg_id, reply = execute(code='x=1', user_expressions=dict(foo='x+1'))
     user_expressions = reply['user_expressions']
-    nt.assert_equal(user_expressions, {u'foo': {
-        u'status': u'ok',
-        u'data': {u'text/plain': u'2'},
-        u'metadata': {},
+    nt.assert_equal(user_expressions, {'foo': {
+        'status': 'ok',
+        'data': {'text/plain': '2'},
+        'metadata': {},
     }})
 
 
@@ -378,7 +375,7 @@ def test_oinfo():
     flush_channels()
 
     msg_id = KC.inspect('a')
-    reply = KC.get_shell_msg(timeout=TIMEOUT)
+    reply = get_reply(KC, msg_id, TIMEOUT)
     validate_message(reply, 'inspect_reply', msg_id)
 
 
@@ -388,7 +385,7 @@ def test_oinfo_found():
     msg_id, reply = execute(code='a=5')
 
     msg_id = KC.inspect('a')
-    reply = KC.get_shell_msg(timeout=TIMEOUT)
+    reply = get_reply(KC, msg_id, TIMEOUT)
     validate_message(reply, 'inspect_reply', msg_id)
     content = reply['content']
     assert content['found']
@@ -403,7 +400,7 @@ def test_oinfo_detail():
     msg_id, reply = execute(code='ip=get_ipython()')
 
     msg_id = KC.inspect('ip.object_inspect', cursor_pos=10, detail_level=1)
-    reply = KC.get_shell_msg(timeout=TIMEOUT)
+    reply = get_reply(KC, msg_id, TIMEOUT)
     validate_message(reply, 'inspect_reply', msg_id)
     content = reply['content']
     assert content['found']
@@ -416,7 +413,7 @@ def test_oinfo_not_found():
     flush_channels()
 
     msg_id = KC.inspect('dne')
-    reply = KC.get_shell_msg(timeout=TIMEOUT)
+    reply = get_reply(KC, msg_id, TIMEOUT)
     validate_message(reply, 'inspect_reply', msg_id)
     content = reply['content']
     assert not content['found']
@@ -428,7 +425,7 @@ def test_complete():
     msg_id, reply = execute(code="alpha = albert = 5")
 
     msg_id = KC.complete('al', 2)
-    reply = KC.get_shell_msg(timeout=TIMEOUT)
+    reply = get_reply(KC, msg_id, TIMEOUT)
     validate_message(reply, 'complete_reply', msg_id)
     matches = reply['content']['matches']
     for name in ('alpha', 'albert'):
@@ -439,7 +436,7 @@ def test_kernel_info_request():
     flush_channels()
 
     msg_id = KC.kernel_info()
-    reply = KC.get_shell_msg(timeout=TIMEOUT)
+    reply = get_reply(KC, msg_id, TIMEOUT)
     validate_message(reply, 'kernel_info_reply', msg_id)
 
 
@@ -450,7 +447,7 @@ def test_connect_request():
     return msg['header']['msg_id']
 
     msg_id = KC.kernel_info()
-    reply = KC.get_shell_msg(timeout=TIMEOUT)
+    reply = get_reply(KC, msg_id, TIMEOUT)
     validate_message(reply, 'connect_reply', msg_id)
 
 
@@ -459,7 +456,7 @@ def test_comm_info_request():
     if not hasattr(KC, 'comm_info'):
         raise SkipTest()
     msg_id = KC.comm_info()
-    reply = KC.get_shell_msg(timeout=TIMEOUT)
+    reply = get_reply(KC, msg_id, TIMEOUT)
     validate_message(reply, 'comm_info_reply', msg_id)
 
 
@@ -485,7 +482,7 @@ def test_is_complete():
     flush_channels()
 
     msg_id = KC.is_complete("a = 1")
-    reply = KC.get_shell_msg(timeout=TIMEOUT)
+    reply = get_reply(KC, msg_id, TIMEOUT)
     validate_message(reply, 'is_complete_reply', msg_id)
 
 def test_history_range():
@@ -495,7 +492,7 @@ def test_history_range():
     reply_exec = KC.get_shell_msg(timeout=TIMEOUT)
 
     msg_id = KC.history(hist_access_type = 'range', raw = True, output = True, start = 1, stop = 2, session = 0)
-    reply = KC.get_shell_msg(timeout=TIMEOUT)
+    reply = get_reply(KC, msg_id, TIMEOUT)
     validate_message(reply, 'history_reply', msg_id)
     content = reply['content']
     assert len(content['history']) == 1
@@ -507,7 +504,7 @@ def test_history_tail():
     reply_exec = KC.get_shell_msg(timeout=TIMEOUT)
 
     msg_id = KC.history(hist_access_type = 'tail', raw = True, output = True, n = 1, session = 0)
-    reply = KC.get_shell_msg(timeout=TIMEOUT)
+    reply = get_reply(KC, msg_id, TIMEOUT)
     validate_message(reply, 'history_reply', msg_id)
     content = reply['content']
     assert len(content['history']) == 1
@@ -519,7 +516,7 @@ def test_history_search():
     reply_exec = KC.get_shell_msg(timeout=TIMEOUT)
 
     msg_id = KC.history(hist_access_type = 'search', raw = True, output = True, n = 1, pattern = '*', session = 0)
-    reply = KC.get_shell_msg(timeout=TIMEOUT)
+    reply = get_reply(KC, msg_id, TIMEOUT)
     validate_message(reply, 'history_reply', msg_id)
     content = reply['content']
     assert len(content['history']) == 1
@@ -535,15 +532,15 @@ def test_stream():
     stdout = KC.iopub_channel.get_msg(timeout=TIMEOUT)
     validate_message(stdout, 'stream', msg_id)
     content = stdout['content']
-    assert content['text'] == u'hi\n'
+    assert content['text'] == 'hi\n'
 
 
 def test_display_data():
     flush_channels()
 
-    msg_id, reply = execute("from IPython.core.display import display; display(1)")
+    msg_id, reply = execute("from IPython.display import display; display(1)")
 
     display = KC.iopub_channel.get_msg(timeout=TIMEOUT)
     validate_message(display, 'display_data', parent=msg_id)
     data = display['content']['data']
-    assert data['text/plain'] == u'1'
+    assert data['text/plain'] == '1'
