@@ -14,13 +14,13 @@ machinery.  This should thus be thought of as scaffolding.
 # Copyright (c) IPython Development Team.
 # Distributed under the terms of the Modified BSD License.
 
-from __future__ import print_function
-
 import os
 import sys
 import time
+import warnings
+from threading import local
 
-from zmq.eventloop import ioloop
+from tornado import ioloop
 
 from IPython.core.interactiveshell import (
     InteractiveShell, InteractiveShellABC
@@ -32,22 +32,29 @@ from IPython.core.error import UsageError
 from IPython.core.magics import MacroToEdit, CodeMagics
 from IPython.core.magic import magics_class, line_magic, Magics
 from IPython.core import payloadpage
-from IPython.core.usage import default_gui_banner
+from IPython.core.usage import default_banner
 from IPython.display import display, Javascript
 from ipykernel import (
     get_connection_file, get_connection_info, connect_qtconsole
 )
 from IPython.utils import openpy
 from ipykernel.jsonutil import json_clean, encode_images
-from IPython.utils.process import arg_split
+from IPython.utils.process import arg_split, system
 from ipython_genutils import py3compat
-from ipython_genutils.py3compat import unicode_type
-from traitlets import Instance, Type, Dict, CBool, CBytes, Any
-from IPython.utils.warn import error
+from traitlets import (
+    Instance, Type, Dict, CBool, CBytes, Any, default, observe
+)
 from ipykernel.displayhook import ZMQShellDisplayHook
-from ipykernel.datapub import ZMQDataPublisher
-from jupyter_client.session import extract_header
-from jupyter_client.session import Session
+
+from jupyter_core.paths import jupyter_runtime_dir
+from jupyter_client.session import extract_header, Session
+
+try:
+    # available since ipyparallel 5.0.0
+    from ipyparallel.engine.datapub import ZMQDataPublisher
+except ImportError:
+    # Deprecated since ipykernel 4.3.0
+    from ipykernel.datapub import ZMQDataPublisher
 
 #-----------------------------------------------------------------------------
 # Functions and classes
@@ -61,6 +68,11 @@ class ZMQDisplayPublisher(DisplayPublisher):
     parent_header = Dict({})
     topic = CBytes(b'display_data')
 
+    # thread_local:
+    # An attribute used to ensure the correct output message
+    # is processed. See ipykernel Issue 113 for a discussion.
+    _thread_local = Any()
+
     def set_parent(self, parent):
         """Set the parent for outbound messages."""
         self.parent_header = extract_header(parent)
@@ -70,27 +82,128 @@ class ZMQDisplayPublisher(DisplayPublisher):
         sys.stdout.flush()
         sys.stderr.flush()
 
-    def publish(self, data, metadata=None, source=None):
+    @default('_thread_local')
+    def _default_thread_local(self):
+        """Initialize our thread local storage"""
+        return local()
+
+    @property
+    def _hooks(self):
+        if not hasattr(self._thread_local, 'hooks'):
+            # create new list for a new thread
+            self._thread_local.hooks = []
+        return self._thread_local.hooks
+
+    def publish(self, data, metadata=None, source=None, transient=None,
+        update=False,
+    ):
+        """Publish a display-data message
+
+        Parameters
+        ----------
+        data: dict
+            A mime-bundle dict, keyed by mime-type.
+        metadata: dict, optional
+            Metadata associated with the data.
+        transient: dict, optional, keyword-only
+            Transient data that may only be relevant during a live display,
+            such as display_id.
+            Transient data should not be persisted to documents.
+        update: bool, optional, keyword-only
+            If True, send an update_display_data message instead of display_data.
+        """
         self._flush_streams()
         if metadata is None:
             metadata = {}
+        if transient is None:
+            transient = {}
         self._validate_data(data, metadata)
         content = {}
         content['data'] = encode_images(data)
         content['metadata'] = metadata
+        content['transient'] = transient
+
+        msg_type = 'update_display_data' if update else 'display_data'
+
+        # Use 2-stage process to send a message,
+        # in order to put it through the transform
+        # hooks before potentially sending.
+        msg = self.session.msg(
+            msg_type, json_clean(content),
+            parent=self.parent_header
+        )
+
+        # Each transform either returns a new
+        # message or None. If None is returned,
+        # the message has been 'used' and we return.
+        for hook in self._hooks:
+            msg = hook(msg)
+            if msg is None:
+                return
+
         self.session.send(
-            self.pub_socket, u'display_data', json_clean(content),
-            parent=self.parent_header, ident=self.topic,
+            self.pub_socket, msg, ident=self.topic,
         )
 
     def clear_output(self, wait=False):
+        """Clear output associated with the current execution (cell).
+
+        Parameters
+        ----------
+        wait: bool (default: False)
+            If True, the output will not be cleared immediately,
+            instead waiting for the next display before clearing.
+            This reduces bounce during repeated clear & display loops.
+
+        """
         content = dict(wait=wait)
         self._flush_streams()
         self.session.send(
-            self.pub_socket, u'clear_output', content,
+            self.pub_socket, 'clear_output', content,
             parent=self.parent_header, ident=self.topic,
         )
 
+    def register_hook(self, hook):
+        """
+        Registers a hook with the thread-local storage.
+
+        Parameters
+        ----------
+        hook : Any callable object
+
+        Returns
+        -------
+        Either a publishable message, or `None`.
+
+        The DisplayHook objects must return a message from
+        the __call__ method if they still require the
+        `session.send` method to be called after transformation.
+        Returning `None` will halt that execution path, and
+        session.send will not be called.
+        """
+        self._hooks.append(hook)
+
+    def unregister_hook(self, hook):
+        """
+        Un-registers a hook with the thread-local storage.
+
+        Parameters
+        ----------
+        hook: Any callable object which has previously been
+              registered as a hook.
+
+        Returns
+        -------
+        bool - `True` if the hook was removed, `False` if it wasn't
+               found.
+        """
+        try:
+            self._hooks.remove(hook)
+            return True
+        except ValueError:
+            return False
+
+
 @magics_class
 class KernelMagics(Magics):
     #------------------------------------------------------------------------
@@ -143,7 +256,7 @@ class KernelMagics(Magics):
 
         Arguments:
 
-        If arguments are given, the following possibilites exist:
+        If arguments are given, the following possibilities exist:
 
         - The arguments are numbers or pairs of colon-separated numbers (like
           1 4:8 9). These are interpreted as lines of previous input to be
@@ -181,11 +294,11 @@ class KernelMagics(Magics):
         Note that %edit is also available through the alias %ed.
         """
 
-        opts,args = self.parse_options(parameter_s,'prn:')
+        opts,args = self.parse_options(parameter_s, 'prn:')
 
         try:
             filename, lineno, _ = CodeMagics._find_edit_target(self.shell, args, opts, last_call)
-        except MacroToEdit as e:
+        except MacroToEdit:
             # TODO: Implement macro editing over 2 processes.
             print("Macro editing not yet implemented in 2-process model.")
             return
@@ -252,44 +365,31 @@ class KernelMagics(Magics):
         In the simplest case, when called from the most recently launched kernel,
         secondary clients can be connected, simply with:
 
-        $> ipython <app> --existing
+        $> jupyter <app> --existing
 
         """
 
-        from IPython.core.application import BaseIPythonApplication as BaseIPApp
-
-        if BaseIPApp.initialized():
-            app = BaseIPApp.instance()
-            security_dir = app.profile_dir.security_dir
-            profile = app.profile
-        else:
-            profile = 'default'
-            security_dir = ''
-
         try:
             connection_file = get_connection_file()
             info = get_connection_info(unpack=False)
         except Exception as e:
-            error("Could not get connection info: %r" % e)
+            warnings.warn("Could not get connection info: %r" % e)
             return
 
-        # add profile flag for non-default profile
-        profile_flag = "--profile %s" % profile if profile != 'default' else ""
-
-        # if it's in the security dir, truncate to basename
-        if security_dir == os.path.dirname(connection_file):
+        # if it's in the default dir, truncate to basename
+        if jupyter_runtime_dir() == os.path.dirname(connection_file):
             connection_file = os.path.basename(connection_file)
 
 
         print (info + '\n')
         print ("Paste the above JSON into a file, and connect with:\n"
-            "    $> ipython <app> --existing <file>\n"
+            "    $> jupyter <app> --existing <file>\n"
             "or, if you are local, you can connect with just:\n"
-            "    $> ipython <app> --existing {0} {1}\n"
+            "    $> jupyter <app> --existing {0}\n"
             "or even just:\n"
-            "    $> ipython <app> --existing {1}\n"
-            "if this is the most recent IPython session you have started.".format(
-            connection_file, profile_flag
+            "    $> jupyter <app> --existing\n"
+            "if this is the most recent Jupyter kernel you have started.".format(
+            connection_file
             )
         )
 
@@ -302,18 +402,15 @@ class KernelMagics(Magics):
         """
 
         # %qtconsole should imply bind_kernel for engines:
-        try:
-            from IPython.parallel import bind_kernel
-        except ImportError:
-            # technically possible, because parallel has higher pyzmq min-version
-            pass
-        else:
+        # FIXME: move to ipyparallel Kernel subclass
+        if 'ipyparallel' in sys.modules:
+            from ipyparallel import bind_kernel
             bind_kernel()
 
         try:
-            p = connect_qtconsole(argv=arg_split(arg_s, os.name=='posix'))
+            connect_qtconsole(argv=arg_split(arg_s, os.name=='posix'))
         except Exception as e:
-            error("Could not start qtconsole: %r" % e)
+            warnings.warn("Could not start qtconsole: %r" % e)
             return
 
     @line_magic
@@ -329,8 +426,8 @@ class KernelMagics(Magics):
 
         try:
             interval = int(arg_s)
-        except ValueError:
-            raise UsageError("%%autosave requires an integer, got %r" % arg_s)
+        except ValueError as e:
+            raise UsageError("%%autosave requires an integer, got %r" % arg_s) from e
 
         # javascript wants milliseconds
         milliseconds = 1000 * interval
@@ -352,39 +449,46 @@ class ZMQInteractiveShell(InteractiveShell):
     kernel = Any()
     parent_header = Any()
 
-    def _banner1_default(self):
-        return default_gui_banner
+    @default('banner1')
+    def _default_banner1(self):
+        return default_banner
 
     # Override the traitlet in the parent class, because there's no point using
     # readline for the kernel. Can be removed when the readline code is moved
     # to the terminal frontend.
-    colors_force = CBool(True)
     readline_use = CBool(False)
     # autoindent has no meaning in a zmqshell, and attempting to enable it
     # will print a warning in the absence of readline.
     autoindent = CBool(False)
 
     exiter = Instance(ZMQExitAutocall)
-    def _exiter_default(self):
+
+    @default('exiter')
+    def _default_exiter(self):
         return ZMQExitAutocall(self)
 
-    def _exit_now_changed(self, name, old, new):
+    @observe('exit_now')
+    def _update_exit_now(self, change):
         """stop eventloop when exit_now fires"""
-        if new:
-            loop = ioloop.IOLoop.instance()
-            loop.add_timeout(time.time()+0.1, loop.stop)
+        if change['new']:
+            loop = self.kernel.io_loop
+            loop.call_later(0.1, loop.stop)
+            if self.kernel.eventloop:
+                exit_hook = getattr(self.kernel.eventloop, 'exit_hook', None)
+                if exit_hook:
+                    exit_hook(self.kernel)
 
     keepkernel_on_exit = None
 
     # Over ZeroMQ, GUI control isn't done with PyOS_InputHook as there is no
     # interactive input being read; we provide event loop support in ipkernel
-    @staticmethod
-    def enable_gui(gui):
+    def enable_gui(self, gui):
         from .eventloops import enable_gui as real_enable_gui
         try:
             real_enable_gui(gui)
+            self.active_eventloop = gui
         except ValueError as e:
-            raise UsageError("%s" % e)
+            raise UsageError("%s" % e) from e
 
     def init_environment(self):
         """Configure the user's environment."""
@@ -402,6 +506,25 @@ class ZMQInteractiveShell(InteractiveShell):
         super(ZMQInteractiveShell, self).init_hooks()
         self.set_hook('show_in_pager', page.as_hook(payloadpage.page), 99)
 
+    def init_data_pub(self):
+        """Delay datapub init until request, for deprecation warnings"""
+        pass
+
+    @property
+    def data_pub(self):
+        if not hasattr(self, '_data_pub'):
+            warnings.warn("InteractiveShell.data_pub is deprecated outside IPython parallel.",
+                DeprecationWarning, stacklevel=2)
+
+            self._data_pub = self.data_pub_class(parent=self)
+            self._data_pub.session = self.display_pub.session
+            self._data_pub.pub_socket = self.display_pub.pub_socket
+        return self._data_pub
+
+    @data_pub.setter
+    def data_pub(self, pub):
+        self._data_pub = pub
+
     def ask_exit(self):
         """Engage the exit actions."""
         self.exit_now = (not self.keepkernel_on_exit)
@@ -411,15 +534,19 @@ class ZMQInteractiveShell(InteractiveShell):
             )
         self.payload_manager.write_payload(payload)
 
+    def run_cell(self, *args, **kwargs):
+        self._last_traceback = None
+        return super(ZMQInteractiveShell, self).run_cell(*args, **kwargs)
+
     def _showtraceback(self, etype, evalue, stb):
         # try to preserve ordering of tracebacks and print statements
         sys.stdout.flush()
         sys.stderr.flush()
 
         exc_content = {
-            u'traceback' : stb,
-            u'ename' : unicode_type(etype.__name__),
-            u'evalue' : py3compat.safe_unicode(evalue),
+            'traceback' : stb,
+            'ename' : str(etype.__name__),
+            'evalue' : py3compat.safe_unicode(evalue),
         }
 
         dh = self.displayhook
@@ -429,18 +556,12 @@ class ZMQInteractiveShell(InteractiveShell):
         if dh.topic:
             topic = dh.topic.replace(b'execute_result', b'error')
 
-        exc_msg = dh.session.send(dh.pub_socket, u'error', json_clean(exc_content), dh.parent_header, ident=topic)
+        exc_msg = dh.session.send(dh.pub_socket, 'error', json_clean(exc_content),
+                                  dh.parent_header, ident=topic)
 
-        # FIXME - Hack: store exception info in shell object.  Right now, the
-        # caller is reading this info after the fact, we need to fix this logic
-        # to remove this hack.  Even uglier, we need to store the error status
-        # here, because in the main loop, the logic that sets it is being
-        # skipped because runlines swallows the exceptions.
-        exc_content[u'status'] = u'error'
-        self._reply_content = exc_content
-        # /FIXME
-
-        return exc_content
+        # FIXME - Once we rely on Python 3, the traceback is stored on the
+        # exception object, so we shouldn't need to store it here.
+        self._last_traceback = stb
 
     def set_next_input(self, text, replace=False):
         """Send the specified text to the frontend to be presented at the next
@@ -457,7 +578,8 @@ class ZMQInteractiveShell(InteractiveShell):
         self.parent_header = parent
         self.displayhook.set_parent(parent)
         self.display_pub.set_parent(parent)
-        self.data_pub.set_parent(parent)
+        if hasattr(self, '_data_pub'):
+            self.data_pub.set_parent(parent)
         try:
             sys.stdout.set_parent(parent)
         except AttributeError:
@@ -470,14 +592,53 @@ class ZMQInteractiveShell(InteractiveShell):
     def get_parent(self):
         return self.parent_header
 
-    #-------------------------------------------------------------------------
-    # Things related to magics
-    #-------------------------------------------------------------------------
-
     def init_magics(self):
         super(ZMQInteractiveShell, self).init_magics()
         self.register_magics(KernelMagics)
         self.magics_manager.register_alias('ed', 'edit')
 
+    def init_virtualenv(self):
+        # Overridden not to do virtualenv detection, because it's probably
+        # not appropriate in a kernel. To use a kernel in a virtualenv, install
+        # it inside the virtualenv.
+        # https://ipython.readthedocs.io/en/latest/install/kernel_install.html
+        pass
+
+    def system_piped(self, cmd):
+        """Call the given cmd in a subprocess, piping stdout/err
+
+        Parameters
+        ----------
+        cmd : str
+          Command to execute (can not end in '&', as background processes are
+          not supported.  Should not be a command that expects input
+          other than simple text.
+        """
+        if cmd.rstrip().endswith('&'):
+            # this is *far* from a rigorous test
+            # We do not support backgrounding processes because we either use
+            # pexpect or pipes to read from.  Users can always just call
+            # os.system() or use ip.system=ip.system_raw
+            # if they really want a background process.
+            raise OSError("Background processes not supported.")
+
+        # we explicitly do NOT return the subprocess status code, because
+        # a non-None value would trigger :func:`sys.displayhook` calls.
+        # Instead, we store the exit_code in user_ns.
+        # Also, protect system call from UNC paths on Windows here too
+        # as is done in InteractiveShell.system_raw
+        if sys.platform == 'win32':
+            cmd = self.var_expand(cmd, depth=1)
+            from IPython.utils._process_win32 import AvoidUNCPath
+            with AvoidUNCPath() as path:
+                if path is not None:
+                    cmd = 'pushd %s &&%s' % (path, cmd)
+                self.user_ns['_exit_code'] = system(cmd)
+        else:
+            self.user_ns['_exit_code'] = system(self.var_expand(cmd, depth=1))
+
+    # Ensure new system_piped implementation is used
+    system = system_piped
+
 
 InteractiveShellABC.register(ZMQInteractiveShell)
